@@ -147,6 +147,118 @@ void saphire_get_quoted_fname_readline_ver(char* file, string_obj* ret)
 
 
 static string_obj* gEditingDir;
+static BOOL read_one_argument(char* p, string_obj* buf)
+{
+    /// 単語の区切り記号かどうかのテーブル 127以下なら
+    static unsigned char table[] = { 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+        1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 
+        1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0 
+    };
+
+    BOOL squote = FALSE;
+    BOOL dquote = FALSE;
+    while(*p && (p - rl_line_buffer < rl_point)) {
+        /// 空文字
+        if(!squote && !dquote && *p == '"' && *(p+1) == '"') {
+            p+=2;
+            string_put(buf, "");
+        }
+        else if(!dquote && !squote && *p == '\'' && *(p+1) == '\'') {
+            p+=2;
+            string_put(buf, "");
+        }
+        else if(*p == '\\' && *(p+1) == 't') {
+            string_push_back2(buf, '\t');
+
+            p+=2;
+        }
+        else if(*p == '\\' && *(p+1) == 'n') {
+            string_push_back2(buf, '\n');
+
+            p+=2;
+        }
+        else if(*p == '\\' && *(p+1) == 'r') {
+            string_push_back2(buf, '\r');
+
+            p+=2;
+        }
+        else if(!squote && !dquote && *p == '\\') {
+            p++;
+
+            if(*p != 0) {
+                string_push_back2(buf, *p);
+                p++;
+            }
+            else {
+                break;
+            }
+        }
+        else if(!dquote && *p == '\'') {
+            p++;
+            squote = !squote;
+        }
+        else if(!squote && *p == '"') {
+            dquote = !dquote;
+            p++;
+        }
+        else if(squote || dquote) {
+            if(*p == 0) {
+                break;
+            }
+            else {
+                string_push_back2(buf, *p);
+
+                p++;
+            }
+        }
+        else if(((unsigned char)*p) > 127) {
+            string_push_back2(buf, *p);
+            p++;
+        }
+        else if(table[*p]) {
+            string_put(buf, "");
+            p++;
+        }
+        else {
+            string_push_back2(buf, *p);
+            p++;
+        }
+    }
+}
+
+void parentname2(char* result, char* path)
+{
+    char* p;
+
+    if(*path == 0) {
+        strcpy(result, "");
+        return;
+    }
+    
+    if(strcmp(path, "/") == 0) {
+        strcpy(result, "/");
+        return;
+    }
+
+    for(p=path + strlen(path)-1; p != path-1; p--) {
+        if(*p == '/' ) {
+            memcpy(result, path, p-path);
+            result[p-path] = '/';
+            result[p-path+1] = 0;
+            
+            return;
+        }
+    }
+
+    strcpy(result, "");
+}
+
 
 #if defined(HAVE_MIGEMO_H)
 #include <migemo.h>
@@ -353,120 +465,8 @@ char* readline_file_completion_migemo_match(char* text, char* file)
     return NULL;
 }
 
-static BOOL read_one_argument(char* p, string_obj* buf)
-{
-    /// 単語の区切り記号かどうかのテーブル 127以下なら
-    static unsigned char table[] = { 
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-        1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 
-        1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0 
-    };
 
-    BOOL squote = FALSE;
-    BOOL dquote = FALSE;
-    while(*p && (p - rl_line_buffer < rl_point)) {
-        /// 空文字
-        if(!squote && !dquote && *p == '"' && *(p+1) == '"') {
-            p+=2;
-            string_put(buf, "");
-        }
-        else if(!dquote && !squote && *p == '\'' && *(p+1) == '\'') {
-            p+=2;
-            string_put(buf, "");
-        }
-        else if(*p == '\\' && *(p+1) == 't') {
-            string_push_back2(buf, '\t');
-
-            p+=2;
-        }
-        else if(*p == '\\' && *(p+1) == 'n') {
-            string_push_back2(buf, '\n');
-
-            p+=2;
-        }
-        else if(*p == '\\' && *(p+1) == 'r') {
-            string_push_back2(buf, '\r');
-
-            p+=2;
-        }
-        else if(!squote && !dquote && *p == '\\') {
-            p++;
-
-            if(*p != 0) {
-                string_push_back2(buf, *p);
-                p++;
-            }
-            else {
-                break;
-            }
-        }
-        else if(!dquote && *p == '\'') {
-            p++;
-            squote = !squote;
-        }
-        else if(!squote && *p == '"') {
-            dquote = !dquote;
-            p++;
-        }
-        else if(squote || dquote) {
-            if(*p == 0) {
-                break;
-            }
-            else {
-                string_push_back2(buf, *p);
-
-                p++;
-            }
-        }
-        else if(((unsigned char)*p) > 127) {
-            string_push_back2(buf, *p);
-            p++;
-        }
-        else if(table[*p]) {
-            string_put(buf, "");
-            p++;
-        }
-        else {
-            string_push_back2(buf, *p);
-            p++;
-        }
-    }
-}
-
-void parentname2(char* result, char* path)
-{
-    char* p;
-
-    if(*path == 0) {
-        strcpy(result, "");
-        return;
-    }
-    
-    if(strcmp(path, "/") == 0) {
-        strcpy(result, "/");
-        return;
-    }
-
-    for(p=path + strlen(path)-1; p != path-1; p--) {
-        if(*p == '/' ) {
-            memcpy(result, path, p-path);
-            result[p-path] = '/';
-            result[p-path+1] = 0;
-            
-            return;
-        }
-    }
-
-    strcpy(result, "");
-}
-
-
-char* readline_file_completion(const char* text, int rstat)
+char* readline_file_completion_migemo(const char* text, int rstat)
 {
     /// クォートを除外 ///
     char* text2 = MALLOC(strlen(text) +1);
@@ -574,7 +574,26 @@ char* readline_file_completion(const char* text, int rstat)
                         strcat(buf, entry->d_name);
                         stat(buf, &_stat);
 
+#if defined(__DARWIN__)
+                        {
+                        char* src = entry->d_name;
+                        char* des = MALLOC(strlen(src)*2);
+                        int des_size = strlen(src)*2;
+                        if(is_all_ascii(src)) {
+                            strcpy(des, src);
+                        }
+                        else {
+                            /// コンバート ///
+                            if(kanji_convert(src, des, des_size, kUtf8Mac, kUtf8) < 0) {
+                                strcpy(des, src);
+                            }
+                        }
+                        strcpy(buf, des);
+                        }
+#else
                         strcpy(buf, entry->d_name);
+#endif
+
                         if(S_ISDIR(_stat.st_mode)) {
                             strcat(buf, "/");
                         }
@@ -604,7 +623,25 @@ char* readline_file_completion(const char* text, int rstat)
                         strcat(buf, entry->d_name);
                         stat(buf, &_stat);
 
+#if defined(__DARWIN__)
+                        {
+                        char* src = entry->d_name;
+                        char* des = MALLOC(strlen(src)*2);
+                        int des_size = strlen(src)*2;
+                        if(is_all_ascii(src)) {
+                            strcpy(des, src);
+                        }
+                        else {
+                            /// コンバート ///
+                            if(kanji_convert(src, des, des_size, kUtf8Mac, kUtf8) < 0) {
+                                strcpy(des, src);
+                            }
+                        }
+                        strcpy(buf, des);
+                        }
+#else
                         strcpy(buf, entry->d_name);
+#endif
                         if(S_ISDIR(_stat.st_mode)) {
                             strcat(buf, "/");
                         }
@@ -794,6 +831,277 @@ char* readline_env_completion(const char* text, int stat)
     return NULL;
 }
 
+char* readline_file_completion(const char* text, int rstat)
+{
+    /// クォートを除外 ///
+    char* text2 = MALLOC(strlen(text) +1);
+    char* _p = (char*)text;
+    char* _p2 = text2;
+
+    while(*_p) {
+        if(*_p == '\\') {
+            _p++;
+        }
+        else {
+            *_p2++ = *_p++;
+        }
+    }
+    *_p2 = 0;
+
+    static int index, wordlen;
+
+    /// 初期化 ///
+    if(rstat == 0) {
+        wordlen = strlen(text2);
+        index = 0;
+
+        // ディレクトリのパートを得る
+        string_put(gEditingDir, "");
+        read_one_argument(rl_line_buffer, gEditingDir);
+        char* tmp = MALLOC(string_length(gEditingDir) + 1);
+        parentname2(tmp, string_c_str(gEditingDir));
+        string_put(gEditingDir, tmp);
+        FREE(tmp);
+
+        /// チルダから始まっている場合 ///
+        if(string_c_str(gEditingDir)[0] == '~') {
+            char* p = string_c_str(gEditingDir);
+            p++;
+            string_obj* user = STRING_NEW("");
+            while(*p && *p != '/') {
+                string_push_back2(user, *p);
+                p++;
+            }
+
+            /// ユーザー名からホームディレクトリに展開
+            string_obj* gEditingDir2 = STRING_NEW("");
+
+            if(string_length(user) == 0) {
+                char* home = getenv("HOME");
+                if(home) {
+                    string_put(gEditingDir2, home);
+                }
+                else {
+                    struct passwd* pw = getpwuid(getuid());
+                    if(pw) {
+                        string_put(gEditingDir2, pw->pw_dir);
+                    }
+                    else {
+                        string_put(gEditingDir2, "~");
+                    }
+                }
+            }
+            else {
+                struct passwd* pw = getpwnam(string_c_str(user));
+                if(pw) {
+                    string_put(gEditingDir2, pw->pw_dir);
+                }
+                else {
+                    string_put(gEditingDir2, "~");
+                    string_push_back(gEditingDir2, string_c_str(user));
+                }
+            }
+
+            string_push_back(gEditingDir2, p);
+
+            string_delete(user);
+
+            string_put(gEditingDir, string_c_str(gEditingDir2));
+            string_delete(gEditingDir2);
+        }
+
+        /// 補完候補を初期化 ///
+        if(files == NULL) {
+            files = VECTOR_NEW(50);
+        }
+        else {
+            int i;
+            for(i=0; i<vector_size(files); i++) {
+                string_delete(vector_item(files, i));
+            }
+            vector_clear(files);
+        }
+
+        /// 補完候補をディレクトリから読み込み ///
+        if(strcmp(string_c_str(gEditingDir), "") != 0) {
+            DIR* dir = opendir(string_c_str(gEditingDir));
+
+            if(dir) {
+                struct dirent* entry;
+                while(entry = readdir(dir)) {
+                    if(strcmp(entry->d_name, ".") != 0
+                            && strcmp(entry->d_name, "..") != 0)
+                    {
+                        char buf[PATH_MAX];
+
+                        struct stat _stat;
+                        strcpy(buf, string_c_str(gEditingDir));
+                        strcat(buf, entry->d_name);
+                        stat(buf, &_stat);
+
+#if defined(__DARWIN__)
+                        {
+                        char* src = entry->d_name;
+                        char* des = MALLOC(strlen(src)*2);
+                        int des_size = strlen(src)*2;
+                        if(is_all_ascii(src)) {
+                            strcpy(des, src);
+                        }
+                        else {
+                            /// コンバート ///
+                            if(kanji_convert(src, des, des_size, kUtf8Mac, kUtf8) < 0) {
+                                strcpy(des, src);
+                            }
+                        }
+                        strcpy(buf, des);
+                        }
+#else
+                        strcpy(buf, entry->d_name);
+#endif
+                        if(S_ISDIR(_stat.st_mode)) {
+                            strcat(buf, "/");
+                        }
+
+                        vector_add(files, STRING_NEW(buf));
+                    }
+                }
+
+                closedir(dir);
+
+                (void)vector_sort(files,name_sort);
+            }
+        }
+        else {
+            DIR* dir = opendir(".");
+
+            if(dir) {
+                struct dirent* entry;
+                while(entry = readdir(dir)) {
+                    if(strcmp(entry->d_name, ".") != 0
+                            && strcmp(entry->d_name, "..") != 0)
+                    {
+                        char buf[PATH_MAX];
+
+                        struct stat _stat;
+                        strcpy(buf, "./");
+                        strcat(buf, entry->d_name);
+                        stat(buf, &_stat);
+
+#if defined(__DARWIN__)
+                        {
+                        char* src = entry->d_name;
+                        char* des = MALLOC(strlen(src)*2);
+                        int des_size = strlen(src)*2;
+                        if(is_all_ascii(src)) {
+                            strcpy(des, src);
+                        }
+                        else {
+                            /// コンバート ///
+                            if(kanji_convert(src, des, des_size, kUtf8Mac, kUtf8) < 0) {
+                                strcpy(des, src);
+                            }
+                        }
+                        strcpy(buf, des);
+                        }
+#else
+                        strcpy(buf, entry->d_name);
+#endif
+                        if(S_ISDIR(_stat.st_mode)) {
+                            strcat(buf, "/");
+                        }
+                        vector_add(files, STRING_NEW(buf));
+                    }
+                }
+
+                closedir(dir);
+
+                (void)vector_sort(files, name_sort);
+            }
+        }
+    }
+
+    while(index < vector_size(files)) {
+        char* file = string_c_str(vector_item(files, index));
+        index++;
+
+        /// ファイル名の入力がない補完は判定しなくても全てマッチ ///
+        if(strcmp(text2, "") == 0) {
+            char path[PATH_MAX];
+            strcpy(path, file);
+
+            string_obj* tmp2 = STRING_NEW("");
+            saphire_get_quoted_fname_readline_ver(path, tmp2);
+            char* ret = strdup(string_c_str(tmp2));
+            string_delete(tmp2);
+
+            struct stat stat_;
+            strcpy(path, string_c_str(gEditingDir));
+            strcat(path, file);
+            if(stat(path, &stat_) == 0) {
+                if(S_ISDIR(stat_.st_mode)) {
+                    rl_completion_append_character= 0;
+                }
+                else {
+                    rl_completion_append_character= ' ';
+                }
+            }
+            else {
+                rl_completion_append_character= ' ';
+            }
+
+            if(ret) {
+                FREE(text2);
+                return ret;
+            }
+        }
+
+        /// アスキー文字同士の補完 ///
+//        else if(is_all_ascii((char*)text2) && is_all_ascii(file)) {
+        else {
+            if(!strncmp(text2, file, wordlen)) {
+                char path[PATH_MAX];
+                strcpy(path, file);
+
+                string_obj* tmp2 = STRING_NEW("");
+                saphire_get_quoted_fname(path, tmp2);
+                char* ret = strdup(string_c_str(tmp2));
+                string_delete(tmp2);
+
+                strcpy(path, string_c_str(gEditingDir));
+                strcat(path, file);
+                struct stat stat_;
+                if(stat(path, &stat_) == 0) {
+                    if(S_ISDIR(stat_.st_mode)) {
+                        rl_completion_append_character= 0;
+                    }
+                    else {
+                        rl_completion_append_character= ' ';
+                    }
+                }
+                else {
+                    rl_completion_append_character= ' ';
+                }
+
+                if(ret) { 
+                    FREE(text2);
+                    return ret; 
+                }
+            }
+        }
+    }
+
+    if(index == 0 && vector_size(files) == 0) {
+        index ++;
+        FREE(text2);
+        rl_completion_append_character= 0;
+        return strdup("");
+    }
+
+
+    FREE(text2);
+    return NULL;
+}
+
 char** readline_on_complete(const char* text, int start, int end)
 {
 //printf("\n\ntext (%s) start %d end %d rl_line_buffer %s\n\n", text, start, end, rl_line_buffer);
@@ -1057,6 +1365,7 @@ char** readline_on_complete(const char* text, int start, int end)
         else if(*p == '/') {
             env = FALSE;
             user = FALSE;
+            program = FALSE;
             p++;
         }
         /// 終了 ///
@@ -1094,10 +1403,13 @@ char** readline_on_complete(const char* text, int start, int end)
         return rl_completion_matches(text, readline_program_completion);
     }
 #if defined(HAVE_MIGEMO_H)
+    else {
+        return rl_completion_matches(text, readline_file_completion_migemo);
+    }
+#else
     else {
         return rl_completion_matches(text, readline_file_completion);
     }
-
 #endif
 
     return NULL;
