@@ -21,6 +21,11 @@
 //////////////////////////////////////////////////////////
 // 宣言
 //////////////////////////////////////////////////////////
+string_obj* gAncestalMemoryPipeIn;
+    // トップレベルの内部コマンド用のパイプ
+string_obj* gAncestalMemoryPipeOut;
+    // トップレベルの内部コマンド用のパイプ
+
 char gTmpDir[PATH_MAX];
     // 一時ファイルの置き場
 
@@ -89,6 +94,8 @@ char* gStatmentKindStrs[kCommand+1] = {
     "yeild",
     "each",
     "foreach",
+    "abs",
+    "fabs",
 //    "parse",
     "inner_command",
     "command"
@@ -2664,7 +2671,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
 
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "1.1.3", 1);
+    setenv("SAPHIRE_VERSION", "1.1.6", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
